# -*- coding: utf-8 -*-

from awstt.worker.scanner import Scanner
from awstt.worker.scanners.ami import AMIScanner
from awstt.worker.scanners.autoscaling import AutoScalingScanner
from awstt.worker.scanners.cloudformation import CloudFormationScanner
from awstt.worker.scanners.cloudfront import CloudFrontScanner
from awstt.worker.scanners.cloudtrail import CloudTrailScanner
from awstt.worker.scanners.cloudwatch_alarm import CloudWatchAlarmScanner
from awstt.worker.scanners.cloudwatch_rules import CloudWatchRulesScanner
from awstt.worker.scanners.dynamodb import DynamoDBScanner
from awstt.worker.scanners.ebs import EBSScanner
from awstt.worker.scanners.ec2 import EC2Scanner
from awstt.worker.scanners.efs import EFSScanner
from awstt.worker.scanners.eip import EIPScanner
from awstt.worker.scanners.elasticache_cluster import ElastiCacheClusterScanner
from awstt.worker.scanners.elasticache_snapshot import ElastiCacheSnapshotScanner
from awstt.worker.scanners.elasticsearch import ElasticsearchScanner
from awstt.worker.scanners.elb import ELBScanner
from awstt.worker.scanners.emr_cluster import EMRClusterScanner
from awstt.worker.scanners.emr_studio import EMRStudioScanner
from awstt.worker.scanners.eni import ENIScanner
from awstt.worker.scanners.iam_policy import IAMPolicyScanner
from awstt.worker.scanners.iam_role import IAMRoleScanner
from awstt.worker.scanners.iam_user import IAMUserScanner
from awstt.worker.scanners.internet_gateway import InternetGatewayScanner
from awstt.worker.scanners.kms import KMSScanner
from awstt.worker.scanners.lambda_function import LambdaScanner
from awstt.worker.scanners.lightsail import LightsailScanner
from awstt.worker.scanners.nat_gateway import NATGatewayScanner
from awstt.worker.scanners.network_acl import NetworkACLScanner
from awstt.worker.scanners.opensearch import OpenSearchScanner
from awstt.worker.scanners.rds_cluster import RDSClusterScanner
from awstt.worker.scanners.rds_instance import RDSInstanceScanner
from awstt.worker.scanners.routetable import RouteTableScanner
from awstt.worker.scanners.s3_bucket import S3BucketScanner
from awstt.worker.scanners.security_group import SecurityGroupScanner
from awstt.worker.scanners.snapshot import SnapshotScanner
from awstt.worker.scanners.subnet import SubnetScanner
from awstt.worker.scanners.vpc import VPCScanner
from awstt.worker.scanners.vpc_peering import VPCPeeringScanner
from awstt.worker.scanners.vpn_gateway import VPNGatewayScanner
from awstt.worker.tagger import Tagger
from awstt.worker.utils import detect_account_id, detect_partition, detect_regions
